/*
 * create two functions to convert temprature
 * by default numeric value is returned
 */

/*proc fcmp outlib=work.myfunc.convert;;*/
/*  deletefunc f2c;*/
/*  deletefunc c2f;*/
/*quit;*/

proc fcmp outlib=work.myfunc.convert;
  function f2c(fahrenheit);
    celsius =  (fahrenheit - 32) * (5 / 9);
    return (celsius);
  endsub;
  function c2f(celsius);
    fahrenheit = 32 + (celsius * (9 / 5));
    return (fahrenheit);
  endsub;
quit;

/*
 * define additional function returning a char value
 * will also search CMPLIB if defined
 */
proc fcmp inlib=work.myfunc outlib=work.myfunc.convert;
  function f2c_c(fahrenheit) $ 8;
    celsius =  f2c(fahrenheit);
    return ( cats(celsius, unicode("\u00b0"), "C") );
  endsub;

quit;

/*
 * show compiled functions
 * can also be written to SAS data set
 */
ods trace on;
ods output CMPSUBList=func_routine_list;
proc fcmp inlib=work.myfunc listfuncs ;
quit;
ods trace off;


/*
 * define search path for functions
 */
options cmplib=(work.myfunc _displayloc_);

/*
 * use functions in various ways
 */
data celsius_fahrenheit;
  do c = 0 to 36;
    f = c2f(c);
    output;
  end;
run;

proc sql;
  create table sql_celsius_fahrenheit as
  select
    *
    , f2c(f) as celsius2
    , f2c_c(f) as celsius3 length=8
  from
    celsius_fahrenheit
  ;
quit;


%put NOTE: 32f = %sysfunc(f2c_c(32));


/*
 * creating a subroutine
 */
proc fcmp outlib=work.myfunc.string;
   subroutine alpha(in $, rev $, upper $);
      outargs rev, upper;  
      rev = left(reverse(in));
      upper = upcase(in);  
   endsub;
quit;

data myclass;
  set sashelp.class;
  length
    nameRev $ 8
    nameUpper $ 8
  ;
  call alpha(name, nameRev, nameUpper);
  keep name:;
run;

