/*
 * from: http://www.sascommunity.org/wiki/sas_filesystem_toolbox
 *
 * 6. list a hierarchy of folders recursively
 *
 * adapted from jason secosky's paper (p. 5 & sqq) user-written data step functions
 * step 1 : fcmp routine xdir_entries compilation
 */

/* naming convention : functions and routines compiled by proc fcmp are prefixed with 'x' to single them out */
proc fcmp outlib=work.funcs.dir;
  /* new ancillary function : open an entry */

  function xdiropen(dir $);
    length dir $ 256 fref $ 8;

    * maximal size for filepath is set tup to 256, can be increased;
    /* a fileref is assigned to the incoming path */
    rc = filename(fref, dir);

    /* check if path can be accessed ...*/
    if rc = 0 then do;

      /* .. then create a pointer with dopen */
      did = dopen(fref);

      /* and free up fileref */
      rc = filename(fref);
    end;

    /*... else catch exception for inaccessibility */
    else do;
      msg = sysmsg();
      put msg '(xdiropen(' dir= ')';

      /* return code gets a missing value */
      did = .;
    end;

    return(did);
  endsub;

  /* routine xdirclose : closing the entry */

  subroutine xdirclose(did);
    outargs did;
    rc = dclose(did);
    did = .;
  endsub;

  /* main routine xdir_entries : recursive listing */
  /* input : rootpath (char string) */
  /* output : members found below (array character) + number of members found + binary flag */

  subroutine xdir_entries(dir $, files[*] $, n, trunc);
    outargs files, n, trunc;
    length dir entry $ 256;

    /* if flag ("trunc") equals 1 then stop - used for array resizing (might be unnecessary with 9.3+) */
    if trunc then
      return;

    /* entry opened */
    did = xdiropen(dir);

    /* if issue accessing the foldern then exit with error */
    if did <= 0 then
      return;

    /* browsing through every entry found */
    dnum = dnum(did);

    do i = 1 to dnum;

      /* test : is this entrt a mere file ? */
      entry = dread(did, i);
      fid = mopen(did, entry);
      entry = trim(dir) || '/' || entry;

      /* a - yes, entry is a file */
      if fid > 0 then do;
        rc = fclose(fid);

        /* if array dimension is sufficient then we store the pathname */
        if n < dim(files) then do;
          trunc = 0;
          n = n + 1;
          files[n] = entry;
        end;

        /* ... else we increment flag */
        else do;
          trunc = 1;
          call xdirclose(did);
          return;
        end;
      end;
      /* b - entry is not a file but a folder : iterate upon this new root folder */
      else  
        call xdir_entries(entry, files, n, trunc);
    end;

    call xdirclose(did);
    return;
  endsub;
quit;

/*
step 2 : calling the xdir_entries in a datastep
*/

/* modify with your own fcmp catalog */
options cmplib=work.funcs;

/* input parameters : rootpath and maximal size of members (can be increased) */
%let racine=c:\temp;
%let array_size=99999;

/* the table will store the members listed */
data file_list(keep=file);
  /* afiles array assignment ('a' for 'array')*/
  array afiles[&array_size] $ 256 _temporary_;
  dnum = 0;
  trunc = 0;

  /* recursive browsing through the rootpath and store members found into "afiles" */
  call xdir_entries("&racine", afiles, dnum, trunc);

  if trunc then
    put 'ERROR: not enough result array entries. increase array size.';

  /* once array is returned, we write an observation for each value */
  do i = 1 to dnum;
    file=afiles[i];
    output;
  end;
run;