/*
 * Master list of southern states and capitals
 * make many entries to show loading time
 */
data southernstcap_masterlist;
  infile datalines dlm="," dsd truncover;
  input state : $50. capital : $50.;
  output;
  statex = state;
  capitalx = capital;
  do i = 2 to 100000;
    state = cats(statex, i);
    capital = cats(capitalx, i);
    output;
  end;

  drop statex capitalx;

  datalines;
North Carolina, Raleigh
Virginia, Richmond
South Carolina, Columbia
Arkansas, Little Rock
Mississippi, Jackson
Alabama, Montgomery
Louisiana, Baton Rouge
Tennessee, Nashville
Georgia, Atlanta
Florida, Tallahassee
Kentucky, Frankfort
West Virginia, Charleston
;

/* 
 * List of states
 */
data states;
  infile datalines truncover;
  input state $char25.;
  datalines;
Virginia
North Carolina100000
Texas
Alabama
Mississippi
Georgia
Georgia12345
Arkansas
;

/*
Load a lookup table into a hash object using PROC FCMP. Use the FIND method to 
determine if the key value has a match in the hash object. If so, output the value
or return a value of Not Found.

Note: For detailed information regarding object dot programming in the DATA step, 
please refer to SAS Component Objects: Reference.

*/
proc fcmp outlib=work.myfunc.hash;
  function sas_lkupcapital(state $) $ 50;
    startTime = datetime();
    length capital $ 50;
    declare hash myHobj (dataset: "work.southernstcap_masterlist");
    rc=myHobj.definedata("capital");
    rc=myHobj.definekey("state");
    rc=myHobj.definedone();
    endTime = datetime();
    diffTime = endTime - startTime;
    nItems = myHobj.NUM_ITEMS();
    /*
     * find looks for states and if found
     * returns capital
     */
    rc=myHobj.find();
    put diffTime= rc= nItems= state= capital=;

    returnValue = capital;
    if rc = 0 then do;
      returnValue = capital;
    end;
    else do;
      returnValue = "NOT FOUND";
    end;

    return (returnValue);
  endsub;
run;

options cmplib=work.myfunc;

/*
 * Call the user-defined function created by Proc FCMP
 * to return the value of the capital for each state
 * 
 */
data test;
  set states;
  capital_returnval = sas_lkupcapital(state);
run;
